{% extends "base.html" %}

{% block title %}{{ block.super }} - Login{% endblock %}

{% block content %}
    <h1>Please login using your Spil Games account</h1>

    {% if form.errors %}
        <div class="alert alert-error">
            Your username and password didn't match. Please try again.
        </div>
    {% endif %}

    <form class="form-horizontal" method="POST" action=".">
        {% csrf_token %}
        <input type="hidden" name="next" value="{{ next }}" />

        <div class="control-group">
            <label class="control-label" for="username">Username</label>
            <div class="controls">
                <input type="text" id="username" name="username" />
            </div>
        </div>
        <div class="control-group">
            <label class="control-label" for="password">Password</label>
            <div class="controls">
                <input type="password" id="password" name="password" />
            </div>
        </div>

        <div class="form-actions">
            <button type="submit" class="btn btn-primary">Login</button>
        </div>
    </form>

{% endblock %}
