@import "../../less/bootstrap/variables.less";

#job_runner {
    h4 {
        margin-bottom: 30px;
        text-align: center;
    }

    h5 {
        margin: 0px;
    }

    .job-run, .job {
        padding: 10px;
        border-radius: 15px;
        margin-bottom: 15px;

        h5 {
            height: 40px;
            overflow: hidden;
        }

        ul {
            list-style: none;
            margin-left: 0px;
            margin-bottom: 0px;
        }
    }

    .job {
        background: #d9edf7;
    }

    .job-scheduled, .job-in-queue, .job-started {
        background: #d9edf7;
    }

    .job-completed {
        background: #dff0d8;
    }

    .job-completed-with-error {
        background: #f2dede;
    }

    .suspended {
        background: #eee;
    }

    .selected {
        box-shadow: 0 0 0 1px @blueDark inset;
    }

    .jobs .span2:nth-child(3n+1) {
        margin-left: 0px;
    }

    pre {
        font-size: 10px;
    }

    .run-performance-graph {
        height: 120px;
    }

    .filters {
        margin-bottom: 30px;
    }

}

.modal {
    width: 900px;
    margin-left: -450px;
}

// copy from .navbar .nav > li > a
// used to align non links properly in the navbar
.navbar .nav > li > span {
    display: block;
    float: none;
    margin: ((@navbarHeight - @baseLineHeight) / 2) + 1 15px ((@navbarHeight - @baseLineHeight) / 2);
    text-decoration: none;
}
