{% extends "base.html" %}
{% load compress %}

{% block ngapp %}jobrunner{% endblock %}

{% block extrahead %}
    <script type="text/javascript">
        var ws_server = '{{ ws_server }}';
    </script>
{% endblock %}

{% block extrajs %}
    <script type="text/javascript" src="https://www.google.com/jsapi"></script>

    {% compress js %}
        <script type="text/javascript" src="{{ STATIC_URL }}js/bootstrap/bootstrap-modal.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}js/bootstrap/bootstrap-transition.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}js/bootstrap/bootstrap-tab.js"></script>

        <script type="text/javascript" src="{{ STATIC_URL }}js/moment.min.js"></script>

        <script type="text/javascript" src="{{ STATIC_URL }}job_runner/js/app.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}job_runner/js/controllers.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}job_runner/js/models.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}job_runner/js/services.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}job_runner/js/filters.js"></script>
    {% endcompress %}

    <script type="text/javascript">google.load("visualization", "1", {packages:["corechart"]});</script>
{% endblock %}

{% block extracss %}
    <link rel="stylesheet/less" href="{{ STATIC_URL }}job_runner/less/job_runner.less" />
{% endblock %}

{% block bodyid %}job_runner{% endblock %}

{% block content %}
    <div ng-view></div>
{% endblock %}
