{% extends "admin/base.html" %}
{% load i18n grp_tags %}

{% block title %}{{ title }} | {% trans 'Job-Runner admin' %}{% endblock %}

{% block branding %}
    <h1 id="site-name">{% trans 'Job-Runner administration' %}</h1>
{% endblock %}

{% block nav-global %}{% endblock %}
